/*
 * xpkFEAL.library V1.00
 *
 * Copyright (C) 1992 Christian von Roques
 *
 */

char version[]="$VER: xpkFEAL 1.0 (1-Nov-1992)

#include "WordOps.h"
#include "FEAL.h"
#include "FEAL_asm.h"

#define ECB_MODE 0	/* deactivated */
/* ECB: C[i]=FEAL(T[i])  T[i]=IFEAL(C[i]) */

#define CBC1_MODE 1	/* default */
/* CBCj: C[i]=FEAL(T[i]^C[i-j]) T[i]=IFEAL(C[i])^C[i-j] */

/* ... more modes will follow! ... */

#include <proto/exec.h>
#include <exec/memory.h>

#define NO_SUB_PRAGMAS

#include <libraries/xpksub.h>

#define A3000 XPKMF_A3000SPEED


// next upto flags packmem unpackmem packspeed unpackspeed ratio reserved 
XMINFO Mode[] = {
	Mode+1,  4,A3000,0,0,238,244,0,0,"fastest",
	Mode+2,  8,A3000,0,0,171,174,0,0,"fast",
	Mode+3, 16,A3000,0,0,109,109,0,0,"safe",
	Mode+4, 32,A3000,0,0, 63, 63,0,0,"safer",
	NULL,  100,A3000,0,0, 34, 34,0,0,"safest"
};


static struct XpkInfo FealInfo = {
	1,			/* info version */
	1,			/* lib  version */
	2,			/* master vers  */
	0,			/* ModesVersion	*/
	"FEAL",			/* short name   */
	"Fast Encryption ALgorithm 1.0",
				/* long name    */
	"FEAL-N with CBC1.  Password protects data with selectable safety.",
				/* description  */
	'FEAL',			/* 4 letter ID  */
	XPKIF_PK_CHUNK  |	/* flags        */
	XPKIF_UP_CHUNK  |
	XPKIF_MODES     |
	XPKIF_ENCRYPTION|
	XPKIF_NEEDPASSWD|
	XPKIF_MODES,
	0x40000000, /* 1GB */	/* max in chunk */
	0,			/* min in chunk */
	32768,			/* def in chunk */
	"Encrypting",		/* pk message   */
	"Decrypting",		/* up message   */
	"Encrypted",		/* pk past msg  */
	"Decrypted",		/* up past msg  */
	16,			/* def mode     */
	0,			/* pad          */
	&Mode			/* modes        */
};


/*
 * Returns the info structure of our encryptor
 */
struct XpkInfo * __saveds __asm
XpksPackerInfo( void )
{
  return &FealInfo;
}


void __saveds __asm
XpksPackFree( REG __a0 XPARAMS* xpar )
{
  if (xpar->Sub[0]) {
    FreeMem(xpar->Sub[0], sizeof(tPartKeys));
  }
}


/*
 * This forces the next chunk to be decryptable independent from the
 * previous one. This is always the case in FEAL.
 */
long __saveds __asm
XpksPackReset( REG __a0 XPARAMS* xpar )
{
  return XPKERR_OK;
}


void __saveds __asm
XpksUnpackFree( REG __a0 XPARAMS* xpar )
{
  if (xpar->Sub[0]) {
    FreeMem(xpar->Sub[0], sizeof(tPartKeys));
  }
}


/*
 * Encrypt a chunk
 */
long __saveds __asm
XpksPackChunk( REG __a0 XPARAMS *xpar )
{
  Word64 Key = password(xpar->Password),
         T,LastC;
  ULONG  *r = (ULONG*)xpar->InBuf, 
         *w = (ULONG*)xpar->OutBuf;
  LONG   len = xpar->InLen;
  int    N = xpar->Mode;
  UWORD  csum = 0;

  LastC.lft=LastC.rgt=0;

  N &= ~2;
  if (N<=0) N=2;
  if (N>MaxN) N=MaxN;

  if (xpar->OutBufLen < ((xpar->InLen+15) & ~7))
    return XPKERR_SMALLBUF;

  if (!xpar->Sub[0]) {
    if (!(xpar->Sub[0] = AllocMem(sizeof(tPartKeys), MEMF_PUBLIC)))
      return XPKERR_NOMEM;
    MakePartKeys((tPartKeys*)xpar->Sub[0], Key, N);
  }

  /* fill in FEAL-Headerinformation. */
  ((UBYTE*)xpar->OutBuf)[2] = CBC1_MODE;
  ((UBYTE*)xpar->OutBuf)[3] = N;
/*((UWORD*)xpar->OutBuf)[0] = 0; checksum, filled in later */
  w++;	/* skip FEAL-Header */

  while (len >= 8) {
    csum += *r;					/* update checksum	*/
    T.lft = *r++ ^ LastC.lft; 
    T.rgt = *r++ ^ LastC.rgt;
    FEAL_asm(&T, w, (tPartKeys*)xpar->Sub[0], N);
    LastC.lft=*w++; LastC.rgt=*w++;
    len -= 8;
  }

  T.lft = *r++;
  T.rgt = *r++;
  switch (len) {
    case 7:                      T.rgt &= 0xFFFFFF00; break;
    case 6:                      T.rgt &= 0xFFFF0000; break;
    case 5:                      T.rgt &= 0xFF000000; break;
    case 4:                      T.rgt  = 0x00000000; break;
    case 3: T.lft &= 0xFFFFFF00; T.rgt  = 0x00000000; break;
    case 2: T.lft &= 0xFFFF0000; T.rgt  = 0x00000000; break;
    case 1: T.lft &= 0xFF000000; T.rgt  = 0x00000000; break;
    case 0: T.lft  = 0x00000000; T.rgt  = 0x00000000; break;
  }
  /* Padding:  The last Byte in the last Block is     */
  /*           the number of Databytes in that Block. */
  T.rgt |= (unsigned long)len;
  csum += T.lft;				/* update checksum	*/
  T.lft^=LastC.lft; T.rgt^=LastC.rgt;
  FEAL_asm(&T, w, (tPartKeys*)xpar->Sub[0], N);
  LastC.lft=*w++; LastC.rgt=*w++;
  len-=len;

  ((UWORD*)xpar->OutBuf)[0] = csum;		/* fill in checksum	*/
  xpar->OutLen = (UBYTE*)w - (UBYTE*)xpar->OutBuf;

  return XPKERR_OK;
}


/*
 * Decrypt a chunk
 */
long __saveds __asm
XpksUnpackChunk( REG __a0 XPARAMS *xpar )
{
  Word64 Key = password(xpar->Password),
         T,LastC;
  ULONG  *r = (ULONG*)xpar->InBuf, 
         *w = (ULONG*)xpar->OutBuf;
  UBYTE  *wb;
  UWORD  csum;
  LONG   len = xpar->InLen - 4 - 8; /* 4:FEAL-Header, 8:last Block */
  int    N;

  LastC.lft=LastC.rgt=0;

  csum = ((UWORD*)xpar->InBuf)[0];		/* checksum		*/
  N    = ((UBYTE*)xpar->InBuf)[3];		/* Number of rounds	*/
  if ((((UBYTE*)xpar->InBuf)[2]!= CBC1_MODE) ||
      (N > MaxN)) {
    return XPKERR_VERSION;
  }
  r++; /* skip FEAL-Header */

  if (xpar->OutBufLen) /* fixed xpkMaster with !=0 OutBufLen */
    if (xpar->OutBufLen < len)
      return XPKERR_SMALLBUF;

  if (!xpar->Sub[0]) {
    if (!(xpar->Sub[0] = AllocMem(sizeof(tPartKeys), MEMF_PUBLIC)))
      return XPKERR_NOMEM;
    MakePartKeys((tPartKeys*)xpar->Sub[0], Key, N);
    PermPartKeys((tPartKeys*)xpar->Sub[0], N);
  }

  xpar->OutLen = len;

  while (len > 0) {
    FEAL_asm(r, w, (tPartKeys*)xpar->Sub[0], N);
    w[0] ^= LastC.lft;
    w[1] ^= LastC.rgt;
    csum -= w[0];
    LastC.lft = *r++;
    LastC.rgt = *r++;
    w+=2;
    len-=8; 
  }

  FEAL_asm(r, &T, (tPartKeys*)xpar->Sub[0], N);
  r+=2;
  T.lft ^= LastC.lft;  T.rgt ^= LastC.rgt;
  csum -= T.lft;
  len = T.rgt & 0xff;
  xpar->OutLen+=len;    /* Add contents of last Block */
  if (len>=4) {
    *w++ = T.lft;
    len -= 4;
    T.lft = T.rgt;
  }
  wb = (UBYTE*)w;
  switch (len) {
      case 3:  wb[2] = (UBYTE) (T.lft >> 8);
      case 2:  wb[1] = (UBYTE) (T.lft >> 16);
      case 1:  wb[0] = (UBYTE) (T.lft >> 24);
  }

  if (csum == 0) {
    return XPKERR_OK;
  } else {
    return XPKERR_WRONGPW;
  }
}
