		/**************************************
		 *                                    *
		 *   AmigaDOS Handler for XPK files   *
		 *                                    *
		 *       Packet action routines       *
		 *                                    *
		 **************************************/

	/* Includes */

#include "includes.h"
#include "defines.h"
#include "protos.h"

	/* Open internal file according to given lock and name */

   LONG
open_file(struct PacketData  *pd, struct DosPacket  *dp)
{
   struct ServerHandle  *sh;
   LONG error = 0L;

   /* First alloc internal file handle */
   if (!(sh = AllocMem(sizeof(struct ServerHandle),
					       MEMF_PUBLIC | MEMF_CLEAR))) {
      error = ERROR_NO_FREE_STORE;
   } else {
      BYTE *path = &sh->sh_Path[0];

      /* Now get real path from given BSTR */
      if (get_real_path(pd, dp, dp->dp_Arg2, dp->dp_Arg3, path,
						      MAX_PATH_LEN) == -1) {
	 error = ERROR_INVALID_COMPONENT_NAME;
      } else {
	 BPTR fh;
	 LONG mode = dp->dp_Type;

	 /* Open file on real filesystem */
	 if (!(fh = Open(path, mode))) {
	    error = IoErr();
	 } else {

	    /* Init internal file handle */
	    sh->sh_OpenMode   = mode;
	    sh->sh_FileHandle = fh;
	    sh->sh_Position   = 0L;
	    sh->sh_Length     = 0L;
	    sh->sh_FileBuffer = NULL;

	    /* Check open mode */
	    if (mode == MODE_OLDFILE) {
	       struct HandlerData  *hd = pd->pd_HandlerData;
	       LONG xpk_error;

	       /* Try to uncompress file opened before */
	       xpk_error = xpk_unpack((ULONG)XPK_InFH, fh,
				(ULONG)XPK_GetOutBuf, &sh->sh_FileBuffer,
				(ULONG)XPK_GetOutLen, &sh->sh_FileSize,
				(ULONG)XPK_GetOutBufLen, &sh->sh_BufferSize,
				(ULONG)XPK_TaskPri, (LONG)hd->hd_XpkPriority,
				(ULONG)TAG_DONE);
	       DEBUG3("   open_file: buf=$%lx size=%ld xpk_error=%ld",
			     sh->sh_FileBuffer, sh->sh_FileSize, xpk_error);
	       if (xpk_error && xpk_error != XPKERR_NOTPACKED) {

		  /* Get error from XpkUnpack() */
		  if (!(error = IoErr())) {
		     error = ERROR_FILE_NOT_OBJECT;
		  }
	       } else {

		  /* If file already unpacked then seek back to beginning */
		  if (xpk_error == XPKERR_NOTPACKED &&
			      Seek(fh, 0L, (LONG)OFFSET_BEGINNING) == -1L) {
		     error = IoErr();
		  }
	       }
	    }

	    /* If no error then link ServerHandle into list */
	    if (!error) {

	       /* Link new internal file handle lock into list */
	       sh->sh_Next       = pd->pd_HandleList;
	       pd->pd_HandleList = sh;

	       /* Init fh_Arg1 of given file handle */
	       ((struct FileHandle *)BTOC(dp->dp_Arg1))->fh_Arg1 = (LONG)sh;
	    } else {

	       /* Don't forget to close file */
	       Close(fh);
	    }
	 }
      }

      /* If any error then release ServerHandle structure */
      if (error) {
	 FreeMem(sh, sizeof(struct ServerHandle));
      }
   }
   DEBUG1("   open_file: error=%ld", error);
   return error;
}
	/* Close given internal file */

   LONG
close_file(struct PacketData  *pd, struct DosPacket  *dp)
{
   struct ServerHandle  *sh = (struct ServerHandle *)dp->dp_Arg1;
   LONG error = 0L;

   /* First check given file handle */
   if (!sh) {
      error = ERROR_INVALID_LOCK;
   } else {
      struct ServerHandle  *parent_sh = (struct ServerHandle *)
							 &pd->pd_HandleList;
      /* Search internal handle in singely linked list */
      while (parent_sh && parent_sh->sh_Next != sh) {
	 parent_sh = parent_sh->sh_Next;
      }
      if (!parent_sh) {
	 error = ERROR_INVALID_LOCK;
      } else {

	 /* Don't forget to close real file */
	 Close(sh->sh_FileHandle);

	 /* Check open mode */
	 DEBUG2("   close_file: mode=%ld name=`%s'", sh->sh_OpenMode,
							   &sh->sh_Path[0]);
	 if (sh->sh_OpenMode == MODE_OLDFILE) {

	    /* If uncompressed file in memory then release it */
	    DEBUG2("   close_file: sh_FileBuffer=$%lx sh_BufferSize=%ld",
				      sh->sh_FileBuffer, sh->sh_BufferSize);
	    if (sh->sh_FileBuffer) {
	       FreeMem(sh->sh_FileBuffer, sh->sh_BufferSize);
	    }
	 } else {
	    struct HandlerData  *hd = pd->pd_HandlerData;

	    /* Check for auto compression */
	    if (hd->hd_Flags & HANDLER_DATA_FLAG_AUTO_COMPRESSION) {
	       BYTE *path = &sh->sh_Path[0],
		    *temp_path = &pd->pd_Path1[0];
	       LONG xpk_error;

	       /* Create full path of temporary file */
	       strcpy(temp_path, path);
	       create_temp_name(BaseName(temp_path));
	       DEBUG1("   close_file: path=`%s'", path);
	       DEBUG1("   close_file: temp=`%s'", temp_path);

	       /* Pack uncompressed file to temp file */
	       xpk_error = xpk_pack((ULONG)XPK_InName, path,
				(ULONG)XPK_OutName, temp_path,
				(ULONG)XPK_PackMethod, hd->hd_PackMethod,
				(ULONG)XPK_Password, hd->hd_Password,
				(ULONG)XPK_StepDown, STEP_DOWN,
				(ULONG)XPK_TaskPri, (LONG)hd->hd_XpkPriority,
				(ULONG)TAG_DONE);
	       DEBUG2("   close_file: hd_PackMethod=`%s' xpk_error=%ld",
					      hd->hd_PackMethod, xpk_error);
	       /* Check result returned from Xpkpack() */
	       if (xpk_error) {

		  /* If file already packed then delete temporary file */
		  if (xpk_error == XPKERR_PACKED) {
		     if (DeleteFile(temp_path) == DOSFALSE) {
			error = IoErr();
		     }
		  }
	       } else {
		  struct XpkFib  *xpk_fib = &pd->pd_XpkFib;

		  /* Compare packed size of file with unpacked one */
		  if (xpk_examine(xpk_fib, (ULONG)XPK_InName, temp_path,
			      TAG_DONE) || xpk_fib->CLen >= xpk_fib->ULen) {
		     if (DeleteFile(temp_path) == DOSFALSE) {
			error = IoErr();
		     }
		  } else {

		     /* Before renaming compressed file delete original file */
		     if (DeleteFile(path) == DOSFALSE) {
			error = IoErr();
		     } else {
			if (Rename(temp_path, path) == DOSFALSE) {
			   error = IoErr();
			}
		     }
		  }
	       }
	    }
	 }

	 /* Remove handle from list and release memory */
	 parent_sh->sh_Next = sh->sh_Next;
	 FreeMem(sh, sizeof(struct ServerHandle));
      }
   }
   DEBUG2("   close_file: sh=$%lx   error=%ld", sh, error);
   return error;
}
	/* Read data from given internal file */

   LONG
read_file(struct PacketData  *pd, struct DosPacket  *dp, LONG *result_ptr)
{
   struct ServerHandle  *sh = (struct ServerHandle *)dp->dp_Arg1;
   LONG result = -1L, error = 0L;

   /* First check given file handle */
   if (!sh) {
      error = ERROR_INVALID_LOCK;
   } else {
      BYTE *buffer = (BYTE *)dp->dp_Arg2;
      LONG len = dp->dp_Arg3;

      /* Check if packed file already in memory */
      DEBUG1("   read_file: name=`%s'", &sh->sh_Path[0]);
      if (sh->sh_FileBuffer) {
	 LONG pos = sh->sh_Position;

	 /* Read data from file in memory */
	 if ((pos + len) > sh->sh_FileSize) {
	    len = sh->sh_FileSize - pos;
	 }
	 if (len) {
	    CopyMem((BYTE *)sh->sh_FileBuffer + pos, (BYTE *)buffer, len);
	 }
	 result = len;
      } else {

	 /* Read data from real file */
	 if ((result = Read(sh->sh_FileHandle, buffer, len)) == -1L) {
	    error = IoErr();
	 }
      }

      /* Calc new position and length of file */
      if (!error) {
	 sh->sh_Position += len;
	 if (sh->sh_Position > sh->sh_Length) {
	    sh->sh_Length = sh->sh_Position + 1;
	 }
      }
   }

   /* Save result */
   *result_ptr = result;
   DEBUG2("   read_file: result=%ld   error=%ld", result, error);
   return error;
}
	/* Write data to given internal file */

   LONG
write_file(struct PacketData  *pd, struct DosPacket  *dp, LONG *result_ptr)
{
   struct ServerHandle  *sh = (struct ServerHandle *)dp->dp_Arg1;
   LONG result = -1L, error = 0L;

   /* First check given file handle */
   if (!sh) {
      error = ERROR_INVALID_LOCK;
   } else {
      BYTE *buffer = (BYTE *)dp->dp_Arg2;
      LONG len = dp->dp_Arg3;

      /* Write data to real file */
      DEBUG1("   write_file: name=`%s'", &sh->sh_Path[0]);
      if ((result = Write(sh->sh_FileHandle, buffer, len)) == -1L) {
	 error = IoErr();
      } else {

	 /* Calc new position and length of file */
	 sh->sh_Position += len;
	 if (sh->sh_Position > sh->sh_Length) {
	    sh->sh_Length = sh->sh_Position + 1;
	 }
      }
   }

   /* Save result */
   *result_ptr = result;
   DEBUG2("   write_file: result=%ld   error=%ld", result, error);
   return error;
}
	/* Seek in given internal file */

   LONG
seek_file(struct PacketData  *pd, struct DosPacket  *dp, LONG *result_ptr)
{
   struct ServerHandle  *sh = (struct ServerHandle *)dp->dp_Arg1;
   LONG result = -1L, error = 0L;

   /* First check given file handle */
   if (!sh) {
      error = ERROR_INVALID_LOCK;
   } else {
      LONG offset = dp->dp_Arg2, mode = dp->dp_Arg3;

      /* Check if packed file already in memory */
      DEBUG1("   seek_file: name=`%s'", &sh->sh_Path[0]);
      if (sh->sh_FileBuffer) {
	 LONG pos = sh->sh_Position;

	 /* Seek in file in memory to new position  */
	 switch (mode) {
	    case OFFSET_BEGINNING :
	       if (offset < sh->sh_FileSize) {
		  sh->sh_Position = offset;
		  result          = pos;
	       }
	       break;

	    case OFFSET_CURRENT :
	       if ((pos + offset) < sh->sh_FileSize && (pos + offset) >= 0L) {
		  sh->sh_Position += offset;
		  result           = pos;
	       }
	       break;

	    case OFFSET_END :
	       if ((sh->sh_FileSize - offset) >= 0L) {
		  sh->sh_Position = sh->sh_FileSize - offset;
		  result          = pos;
	       }
	       break;
	 }
      } else {

	 /* Seek in real file */
	 if ((result = Seek(sh->sh_FileHandle, offset, mode)) == -1L) {
	    error = IoErr();
	 } else {

	    /* Calc new position and length of file */
	    if ((sh->sh_Position = Seek(sh->sh_FileHandle, 0L,
					    (LONG)OFFSET_CURRENT)) == -1L) {
	       result = -1L;
	       error  = IoErr();
	    } else {
	       if (sh->sh_Position > sh->sh_Length) {
		  sh->sh_Length = sh->sh_Position + 1;
	       }
	    }
	 }
      }
   }

   /* Save result */
   *result_ptr = result;
   DEBUG2("   seek_file: result=%ld   error=%ld", result, error);
   return error;
}
	/* Get volume node of given internal file */

   LONG
get_current_volume(struct PacketData  *pd, struct DosPacket  *dp,
							   LONG *result_ptr)
{
   struct ServerHandle  *sh = (struct ServerHandle *)dp->dp_Arg1;
   LONG result = (LONG)CTOB(pd->pd_HandlerData->hd_VolumeNode), error = 0L;

   /* First check given file handle */
   if (sh) {
      struct MsgPort  *mp = DeviceProc(&sh->sh_Path[0]);

      /* Get volume node (result) and unit number (error) */
      DEBUG2("   get_current_volume: mp=$%lx name=`%s'", mp, &sh->sh_Path[0]);
      if (mp) {
	 result = send_packet(mp, (LONG)ACTION_CURRENT_VOLUME,
		   ((struct FileHandle *)BTOC(sh->sh_FileHandle))->fh_Arg1);
	 error  = IoErr();
      }
   }

   /* Save result */
   *result_ptr = result;
   DEBUG2("   get_current_volume: volume=$%lx   unit=%ld", result, error);
   return error;
}
	/* Create internal file lock according to given lock and name */

   LONG
create_internal_lock(struct PacketData  *pd, struct DosPacket  *dp,
							     BPTR *lock_ptr)
{
   BYTE  *path = &pd->pd_Path1[0];
   SHORT len;
   BPTR  lock = NULL;
   LONG  error = 0L;

   /* First get real path from given BSTR */
   if ((len = get_real_path(pd, dp, dp->dp_Arg1, dp->dp_Arg2, path,
						     MAX_PATH_LEN)) == -1) {
      error = ERROR_INVALID_COMPONENT_NAME;
   } else {
      LONG mode = dp->dp_Arg3;

      /* Check if Workbench process requests a lock */
      if (!len) {

	 /* Alloc and init internal lock */
	 error = build_internal_lock(pd, (BPTR)NULL, mode, &lock);
      } else {
	 BPTR real_lock;

	 /* Get lock to real filesystem */
	 if (!(real_lock = Lock(path, mode))) {
	    error = IoErr();
	 } else {

	    /* Alloc and init internal lock */
	    error = build_internal_lock(pd, real_lock, mode, &lock);
	 }
      }
   }

   /* Finally save lock and return error */
   *lock_ptr = lock;
   DEBUG2("   create_internal_lock: lock=$%lx   error=%ld", lock, error);
   return error;
}
	/* Remove given internal lock from list and release memory */

   LONG
delete_internal_lock(struct PacketData  *pd, struct DosPacket  *dp)
{
   BPTR lock = dp->dp_Arg1;
   LONG error = 0L;

   /* First check for ZERO lock (root of this handler) */
   if (!lock) {
      error = ERROR_INVALID_LOCK;
   } else {
      struct FileLock  *parent_fl;

      /* Search internal lock in singely linked list */
      parent_fl = (struct FileLock *)&pd->pd_LockList;
      while (parent_fl && parent_fl->fl_Link != lock) {
	 parent_fl = (struct FileLock *)BTOC(parent_fl->fl_Link);
      }
      if (!parent_fl) {
	 error = ERROR_INVALID_LOCK;
      } else {
	 struct FileLock  *fl = (struct FileLock *)BTOC(lock);

	 /* Don't forget to release real lock if any */
	 if (fl->fl_Key) {
	    UnLock(fl->fl_Key);
	 }

	 /* Remove lock from list and release memory */
	 parent_fl->fl_Link = fl->fl_Link;
	 DosFreeMem(fl);
      }
   }
   DEBUG2("   delete_internal_lock: lock=$%lx   error=%ld", lock, error);
   return error;
}
	/* Check if given locks are the same */

   LONG
same_lock(struct PacketData  *pd, struct DosPacket  *dp, LONG *result_ptr)
{
   BPTR lock1 = dp->dp_Arg1, lock2 = dp->dp_Arg2;
   LONG result = LOCK_DIFFERENT, error = 0L;

   /* Check for internal ZERO lock */
   if (!lock1 || !lock2) {
      error = ERROR_INVALID_LOCK;
   } else {

      /* Check for equal internal locks */
      if (lock1 == lock2) {
	 result = LOCK_SAME;
      } else {
	 BPTR real_lock1 = get_real_lock(pd, lock1),
	      real_lock2 = get_real_lock(pd, lock2);

	 /* Check for real ZERO lock */
	 if (real_lock1 && real_lock2) {

	    /* Check for equal real locks */
	    if (real_lock1 == real_lock2) {
	       result = LOCK_SAME;
	    } else {
	       struct FileLock  *fl1 = (struct FileLock *)BTOC(real_lock1),
				*fl2 = (struct FileLock *)BTOC(real_lock2);
	       /* Compare volumes and handler tasks */
	       if (fl1->fl_Volume == fl2->fl_Volume &&
					     fl1->fl_Task == fl2->fl_Task) {
		  LONG real_result, real_error;
	       
		  /* Ask real filesystem if same lock */
		  real_result = send_packet(fl1->fl_Task,
			    (LONG)ACTION_SAME_LOCK, real_lock1, real_lock2);
		  real_error  = IoErr();
		  DEBUG2("   same_lock: ACTION_SAME_LOCK -> result=%ld "
				      "error=%ld", real_result, real_error);
		  /* Check if this packet is supported */
		  if (real_error == ERROR_ACTION_NOT_KNOWN) {

		     /* Compare keys as usual */
		     if (fl1->fl_Key == fl2->fl_Key) {
			result = LOCK_SAME;
		     } else {
			result = LOCK_SAME_VOLUME;
		     }
		  } else {

		     /* If no error then return result from ACTION_SAME_LOCK */
		     if (!real_error) {
			result = real_result;
		     } else {
			result = LOCK_SAME_VOLUME;
		     }
		  }
	       }
	    }
	 }
      }
   }

   /* Save result */
   *result_ptr = result;
   DEBUG2("   same_lock: result=%ld   error=%ld", result, error);
   return error;
}
	/* Duplicate internal lock */

   LONG
duplicate_internal_lock(struct PacketData  *pd, struct DosPacket  *dp,
							     BPTR *lock_ptr)
{
   BPTR old_lock = dp->dp_Arg1, lock = NULL;
   LONG error = 0L;

   /* First check given original lock */
   if (!old_lock) {
      error = ERROR_INVALID_LOCK;
   } else {
      struct FileLock  *fl = (struct FileLock *)BTOC(old_lock);
      BPTR real_lock;

      /* Get duplicate of real lock */
      if (!(real_lock = DupLock(fl->fl_Key))) {
	 error = IoErr();
      } else {

	 /* Alloc and init internal lock */
	 error = build_internal_lock(pd, real_lock, fl->fl_Access, &lock);
      }
   }

   /* Finally save lock */
   *lock_ptr = lock;
   DEBUG2("   duplicate_internal_lock: lock=$%lx   error=%ld", lock, error);
   return error;
}
	/* Get parent of internal lock */

   LONG
parent_internal_lock(struct PacketData  *pd, struct DosPacket  *dp,
							     BPTR *lock_ptr)
{
   BPTR old_lock = dp->dp_Arg1, lock = NULL;
   LONG error = 0L;

   /* First check given original lock */
   if (!old_lock) {
      error = ERROR_INVALID_LOCK;
   } else {
      struct FileLock  *fl = (struct FileLock *)BTOC(old_lock);
      BPTR real_lock;

      /* Get parent lock from real filesystem */
      if (!(real_lock = ParentDir(fl->fl_Key))) {
	 error = IoErr();
      } else {

	 /* Alloc and init internal lock */
	 error = build_internal_lock(pd, real_lock, fl->fl_Access, &lock);
      }
   }

   /* Finally save lock */
   *lock_ptr = lock;
   DEBUG2("   parent_internal_lock: lock=$%lx   error=%ld", lock, error);
   return error;
}
	/* Get file info block for given lock and packet type */

   LONG
get_file_info_block(struct PacketData  *pd, struct DosPacket  *dp)
{
   struct FileInfoBlock  *fib = (struct FileInfoBlock *)BTOC(dp->dp_Arg2);
   BPTR real_lock, buffer = dp->dp_Arg2;
   LONG type = dp->dp_Type, error = 0L;

   /* First check if real lock attached to internal lock */
   if (!(real_lock = get_real_lock(pd, dp->dp_Arg1))) {
      error = ERROR_INVALID_LOCK;
   } else {

      /* Get file info block from real filesystem */
      if (send_packet(get_real_port(real_lock), type, real_lock,
						      buffer) == DOSFALSE) {
	 error = IoErr();
      } else {

	 /* If packed file then get real file size */
	 if (fib->fib_DirEntryType < 0L) {
	    struct XpkFib  *xpk_fib = &pd->pd_XpkFib;
	    BYTE *path = &pd->pd_Path1[0];

	    /* Get info from packed file */
	    if (PathName(real_lock, path, 10L)) {
	       if (type == ACTION_EXAMINE_NEXT) {
		  BYTE *base = &pd->pd_Path2[0];

		  /* Append base name to path */
		  BtoCStr(base, (BSTR)CTOB(&fib->fib_FileName[0]),
							(LONG)MAX_PATH_LEN);
		  TackOn(path, base);
	       }
	       DEBUG2("   get_file_info_block: type=$%lx path=`%s'", type,
								      path);
	       /* Get unpacked size of file */
	       if (!xpk_examine(xpk_fib, (ULONG)XPK_InName, path, TAG_DONE)) {
		  fib->fib_Size = xpk_fib->ULen;
	       }
	    }
	    DEBUG2("   get_file_info_block: path=`%s' fib_Size=%ld", path,
							     fib->fib_Size);
	 }
      }
   }
   DEBUG2("   get_file_info_block: fib_FileName=`%s' error=%ld",
	   &((struct FileInfoBlock *)BTOC(buffer))->fib_FileName[1], error);
   return error;
}
	/* Get info data for given lock */

   LONG
get_info_data(struct PacketData  *pd, struct DosPacket  *dp)
{
   BPTR real_lock, lock, buffer;
   LONG error = 0L;

   /* Get lock and buffer according to packet type */
   if (dp->dp_Type == ACTION_DISK_INFO) {
      lock   = NULL;
      buffer = dp->dp_Arg1;
   } else {
      lock   = dp->dp_Arg1;
      buffer = dp->dp_Arg2;
   }

   /* First check if real lock attached to internal lock */
   if (!(real_lock = get_real_lock(pd, lock))) {
      error = ERROR_INVALID_LOCK;
   } else {

      /* Get info data from real filesystem */
      if (send_packet(get_real_port(real_lock), (LONG)ACTION_INFO,
					   real_lock, buffer) == DOSFALSE) {
	 error = IoErr();
      } else {

	 /* For ACTION_DISK_INFO use volume node of this handler */
	 if (dp->dp_Type == ACTION_DISK_INFO) {
	    struct InfoData  *id = (struct InfoData *)BTOC(buffer);

	    id->id_VolumeNode = (BPTR)CTOB(pd->pd_HandlerData->hd_VolumeNode);
	 }
      }
   }
   DEBUG1("   get_info_data: error=%ld", error);
   return error;
}
	/* Create directory according to given lock and name */

   LONG
create_dir(struct PacketData  *pd, struct DosPacket  *dp, BPTR *lock_ptr)
{
   BYTE *path = &pd->pd_Path1[0];
   BPTR lock = NULL;
   LONG error = 0L;

   /* First get real path from given BSTR */
   if (get_real_path(pd, dp, dp->dp_Arg1, dp->dp_Arg2, path,
						      MAX_PATH_LEN) == -1) {
      error = ERROR_INVALID_COMPONENT_NAME;
   } else {

      /* Create new directory */
      if (!(lock = CreateDir(path))) {
	 error = IoErr();
      }
   }

   /* Finally save lock */
   *lock_ptr = lock;
   DEBUG2("   create_dir: lock=$%lx   error=%ld", lock, error);
   return error;
}
	/* Delete object according to given lock and name */

   LONG
delete_object(struct PacketData  *pd, struct DosPacket  *dp)
{
   BYTE *path = &pd->pd_Path1[0];
   LONG error = 0L;

   /* First get real path from given BSTR */
   if (get_real_path(pd, dp, dp->dp_Arg1, dp->dp_Arg2, path,
						      MAX_PATH_LEN) == -1) {
      error = ERROR_INVALID_COMPONENT_NAME;
   } else {

      /* Delete given file */
      if (DeleteFile(path) == DOSFALSE) {
	 error = IoErr();
      }
   }
   DEBUG1("   delete_object: error=%ld", error);
   return error;
}
	/* Rename object according to given locks and names */

   LONG
rename_object(struct PacketData  *pd, struct DosPacket  *dp)
{
   BYTE *spath = &pd->pd_Path1[0], *dpath = &pd->pd_Path2[0];
   LONG error = 0L;

   /* First get real paths from given BSTRs */
   if (get_real_path(pd, dp, dp->dp_Arg1, dp->dp_Arg2, spath,
		   MAX_PATH_LEN) == -1 || (get_real_path(pd, dp, dp->dp_Arg3,
				 dp->dp_Arg4, dpath, MAX_PATH_LEN)) == -1) {
      error = ERROR_INVALID_COMPONENT_NAME;
   } else {

      /* Rename given file */
      if (Rename(spath, dpath) == DOSFALSE) {
	 error = IoErr();
      }
   }
   DEBUG1("   rename_object: error=%ld", error);
   return error;
}
	/* Set protection, comment or date date of given object */

   LONG
set_something(struct PacketData  *pd, struct DosPacket  *dp)
{
   BPTR  lock = dp->dp_Arg2;
   BYTE  *path = &pd->pd_Path1[0];
   SHORT len;
   LONG  error = 0L;

   /* First get real path from given BSTR */
   if ((len = get_real_path(pd, dp, lock, dp->dp_Arg3, path,
						     MAX_PATH_LEN)) == -1) {
      error = ERROR_INVALID_COMPONENT_NAME;
   } else {
      BYTE *path_bstr;

      /* Create BSTR from path */
      if (!(path_bstr = AllocMem((LONG)(len + 1), MEMF_PUBLIC))) {
	 error = ERROR_NO_FREE_STORE;
      } else {
	 struct MsgPort  *mp = DeviceProc(path);

	 /* Build BSTR */
	 *path_bstr = (BYTE)len;
	 strncpy(path_bstr + 1, path, (size_t)len);

	 /* Do setting */
	 DEBUG2("   set_something: type=%ld   mp=$%lx", dp->dp_Type, mp);
	 if (mp) {
	    if (send_packet(mp, dp->dp_Type, dp->dp_Arg1, (LONG)NULL,
				CTOB(path_bstr), dp->dp_Arg4) == DOSFALSE) {
	       error = IoErr();
	    }
	 }
	 FreeMem(path_bstr, (LONG)(len + 1));
      }
   }
   DEBUG1("   set_something: error=%ld", error);
   return error;
}
