		/**************************************
		 *                                    *
		 *   AmigaDOS Handler for XPK files   *
		 *                                    *
		 *       Defines and structures       *
		 *                                    *
		 **************************************/

	/* Defines for handler */

#define PROCESS_STACK_SIZE	4096
#define PROCESS_PRIORITY	0

#define TEMPLATE	"ROOTDIR/A,VOLUMENAME/K,AUTOCOMPRESS/S,STEPDOWN/S,XPKPRIORITY/K,PACKMETHOD/K,PASSWORD/K"

#define MAX_ARGUMENTS		6
#define ARGUMENT_ROOT_DIR	0
#define ARGUMENT_VOLUME_NAME	1
#define ARGUMENT_AUTO_COMPRESS	2
#define ARGUMENT_STEP_DOWN	3
#define ARGUMENT_XPK_PRIORITY	4
#define ARGUMENT_PACK_METHOD	5
#define ARGUMENT_PASSWORD	6

#define DEFAULT_ARUMENT_STRING	""
#define DEFAULT_VOLUME_NAME	"Xpk handler"
#define DEFAULT_XPK_PRIORITY	0
#define DEFAULT_PACK_METHOD	"NUKE"

	/* Defines for packet server */

#define PACKET_SERVER_NAME	"Xpk packet server"

#define MAX_TAGGED_ITEMS	10
#define MAX_PATH_LEN		256

#define TEMP_NAME_FORMAT	"XPK%lx%lx%lx"

#define STEP_DOWN	(hd->hd_Flags & HANDLER_DATA_FLAG_STEP_DOWN ? TRUE : FALSE)

	/* Some macros for AmigaDOS */

#define CTOB(cptr)	((VOID *)((ULONG)(cptr) >> 2))	/* convert C ptr to BCPL ptr */
#define BTOC(bptr)	((APTR)((ULONG)(bptr) << 2))	/* convert BCPL ptr to C ptr */

	/* Defines for kick 2.0 packets */

#ifndef	ACTION_SAME_LOCK

#define LOCK_DIFFERENT		-1
#define LOCK_SAME		0
#define LOCK_SAME_VOLUME	1	/* locks are on same volume */

#define	ACTION_SAME_LOCK	40
#define	ACTION_IS_FILESYSTEM	1027

#endif	/* ACTION_SAME_LOCK */

	/* Debugging macros for FarPrint */

#ifdef DEBUG

#define DEBUG0(f)			SendText(f);
#define DEBUG1(f,a1)			SendText(f,a1);
#define DEBUG2(f,a1,a2)			SendText(f,a1,a2);
#define DEBUG3(f,a1,a2,a3)		SendText(f,a1,a2,a3);
#define DEBUG4(f,a1,a2,a3,a4)		SendText(f,a1,a2,a3,a4);
#define DEBUG5(f,a1,a2,a3,a4,a5)	SendText(f,a1,a2,a3,a4,a5);
#define DEBUG6(f,a1,a2,a3,a4,a5,a6)	SendText(f,a1,a2,a3,a4,a5,a6);

#else /* DEBUG */

#define DEBUG0(f)
#define DEBUG1(f,a1)
#define DEBUG2(f,a1,a2)
#define DEBUG3(f,a1,a2,a3)
#define DEBUG4(f,a1,a2,a3,a4)
#define DEBUG5(f,a1,a2,a3,a4,a5)
#define DEBUG6(f,a1,a2,a3,a4,a5,a6)

#endif /* DEBUG */

	/* Structures */

struct HandshakeMsg {   /* used for handshaking with server processes while startup */
	struct Message  hm_Message;
	LONG	hm_Data;
};
struct ServerHandle {   /* used as file handle for packet server process */
        struct ServerHandle  *sh_Next;	/* Linkage of server handles */
	LONG	sh_OpenMode;		/* mode given to Open() */
	BPTR	sh_FileHandle;		/* file handle of real file */
	ULONG	sh_Position;		/* current position in file */
	ULONG	sh_Length;		/* actual file length */
	APTR	sh_FileBuffer;		/* unpacked file in memory */
	LONG	sh_FileSize;		/* size of unpacked file */
	LONG	sh_BufferSize;		/* size of unpack buffer */
	BYTE	sh_Path[MAX_PATH_LEN + 1];
};
	/* Defines and structure for global data of handler process */

#define HANDLER_DATA_FLAG_AUTO_COMPRESS		(1L << 0)
#define HANDLER_DATA_FLAG_STEP_DOWN		(1L << 1)

struct HandlerData {   /* any global data used by handler process */
	struct DeviceNode    *hd_DeviceNode;		/* device node of handler AmigaDOS created while mounting */
	struct DeviceList    *hd_VolumeNode;		/* volume node of handler to allow workbench to see us too */
	struct Process       *hd_Process;		/* ptr to handler process */
	struct MsgPort       *hd_PacketPort;		/* port for receiving packets from AmigaDOS */
	struct MsgPort       *hd_ReplyPort;		/* reply port for packets sent to packet server */
	struct MsgPort       *hd_ZombiePort;		/* reply port for handshake and zombie msg */
	struct MsgPort       *hd_PacketServerPort;	/* port for packet server packets */
	ULONG	hd_Flags;

	/* filled in from parse_argument_string() */
	BPTR	hd_PathLock;		/* lock created from path argument of startup string */
	LONG	hd_XpkPriority;		/* priority for XpkPack()/XpkUnpack() */
	BYTE	*hd_PackMethod;		/* pack method for XpkPack() */
	BYTE	*hd_Password;		/* password for XpkPack()/XpkUnpack() */
};
	/* Structure for global data of packet server process */

#define PACKET_DATA_FLAG_INHIBIT	(1L << 0)

struct PacketData {   /* any global data used by packet server process */
	struct MsgPort       *pd_PacketPort;		/* port for receiving packets from handler process */
	struct HandlerData   *pd_HandlerData;		/* global data used by handler process received via HandshakeMsg */
	struct ArpBase       *pd_ArpBase;		/* Arp library MUST be opened locally for their resource tracking */
	struct ServerHandle  *pd_HandleList;		/* pointer to first internal file handle in singely linked list */
	BPTR	pd_LockList;				/* pointer to first internal file lock in singely linked list */
	ULONG	pd_Flags;
	BYTE	pd_Path1[MAX_PATH_LEN + 1];
	BYTE	pd_Path2[MAX_PATH_LEN + 1];
	struct XpkFib        pd_XpkFib;			/* needed for XpkExamine() */
};
	/* Externals */

IMPORT struct DosLibrary  *DOSBase;
IMPORT struct Library     *XpkBase;

IMPORT struct TextAttr    topaz60_attr, topaz80_attr;

IMPORT const BYTE *ok;
