		/**************************************
		 *                                    *
		 *   AmigaDOS Handler for XPK files   *
		 *                                    *
		 *          Handler routines          *
		 *                                    *
		 **************************************/

	/* Includes */

#include "includes.h"
#include "defines.h"
#include "protos.h"

	/* Globals */

struct IntuitionBase	*IntuitionBase;
struct ArpBase		*ArpBase;
struct Library		*XpkBase;

	/* Handler process startup routine */

   LONG
handler_process_startup(VOID)
{
   struct Process      *pr = (struct Process *)FindTask(NULL);
   struct MsgPort      *pp = &pr->pr_MsgPort;
   struct Message      *mn;
   struct DosPacket    *dp;
   struct DeviceNode   *dn;
   struct HandlerData  *hd;	/* global data for handler process */
   LONG return_code = RETURN_FAIL;

   /* Wait for startup packet and get data from it */
   WaitPort(pp);
   mn = GetMsg(pp);
   dp = (struct DosPacket *)mn->mn_Node.ln_Name;
   dn = (struct DeviceNode *)BTOC(dp->dp_Arg3);
   DEBUG3("handler_process_startup: mn=$%lx dp=$%lx dn=$%lx", mn, dp, dn);

   /* Allocate resources needed for handler process */
   if (IntuitionBase = OpenLibrary("intuition.library", 0L)) {
      if (!(ArpBase = OpenLibrary(ArpName, ArpVersion))) {
	 error_requester("Can't open "ArpName" V39+");
      } else {
	 if (!(XpkBase = OpenLibrary(XPKNAME, 0L))) {
	    error_requester("Can't open "XPKNAME);
	 } else {
	    if (hd = create_handler_data(pr, dp)) {

	       /* Parse arguments given via startup packet */
	       if (parse_argument_string(hd, dp->dp_Arg2) == FALSE) {
		  error_requester("Error while parsing startup string from "
							      "mount list");
	       } else {

		  /* Start packet server process */
		  if (hd->hd_PacketServerPort = create_process(hd->hd_ZombiePort,
			 &packet_server_process_startup, PACKET_SERVER_NAME,
								(LONG)hd)) {
		     /* Patch us into system, so handler process
		      * will not be respawned for every packet
		      */
		     dn->dn_Task = pp;

		     /* Return startup packet */
		     return_packet(dp, DOSTRUE, dp->dp_Res2);

		     /* Insert volume node and call handler main routine */
		     link_volume_node(hd->hd_VolumeNode);
		     handler_process_main(hd);
		     unlink_volume_node(hd->hd_VolumeNode);
		     return_code = RETURN_OK;

		     /* Remove us from system, so new handler
		      * process can be respawned
		      */
		     dn->dn_Task = NULL;
		  }
	       }

	       /* Free resources */
	       delete_handler_data(hd);
	    }
	    CloseLibrary(XpkBase);
	 }
	 CloseLibrary(ArpBase);
      }
      CloseLibrary(IntuitionBase);
   }

   /* If startup failed then return startup packet with error */
   if (return_code == RETURN_FAIL) {
      return_packet(dp, DOSFALSE, (LONG)ERROR_NO_FREE_STORE);
   } else {

      /* Return all pending packets */
      while (mn = GetMsg(pp)) {
	 return_packet((struct DosPacket *)mn->mn_Node.ln_Name, DOSFALSE,
					    (LONG)ERROR_DEVICE_NOT_MOUNTED);
      }
   }

   /* Unloading handler code -> need FORBID */
   DEBUG0("handler_process_startup: unloading handler code");
   Forbid();
   UnLoadSeg(dn->dn_SegList);
   dn->dn_SegList = NULL;

   return return_code;
}
	/* Handler process main routine */

   STATIC VOID
handler_process_main(struct HandlerData  *hd)
{
   struct MsgPort  *pp = &hd->hd_Process->pr_MsgPort, *zp = hd->hd_ZombiePort;
   LONG sigf_zombie, sigf_packet, sig_mask;
   BOOL quit_flag = FALSE;

   /* Build signal masks */
   sigf_zombie = 1L << zp->mp_SigBit;
   sigf_packet = 1L << pp->mp_SigBit;
   sig_mask    = sigf_zombie | sigf_packet;

   /* Packet loop */
   do {
      LONG signals;

      /* Wait for AmigaDOS packet or zombie message signals */
      signals = Wait(sig_mask);

      /* Zombie message from server? */
      if (signals & sigf_zombie) {
	 struct ZombieMsg  *zm;

	 /* Get zombie message from packet server process and quit */
	 if (zm = (struct ZombieMsg *)GetMsg(zp)) {
	    FreeMem(zm, sizeof(struct ZombieMsg));
	    quit_flag = TRUE;
	    DEBUG0(" handler_process_main: zombie msg received");
	 }
      }

      /* New dos packet? */
      if (quit_flag == FALSE && (signals & sigf_packet)) {
	 struct Message  *mn;
	 BOOL keepon = TRUE;

	 /* Send all packets to packet server or reject them */
	 while (keepon == TRUE && (mn = GetMsg(pp))) {
	    struct DosPacket  *dp = (struct DosPacket *)mn->mn_Node.ln_Name;

	    /* Check packet type */
	    DEBUG2(" handler_process_main: dp=$%lx dp_Type=%ld", dp,
							       dp->dp_Type);
	    if (dp->dp_Type == ACTION_DIE) {

	       /* Send packet synchronously to packet server port */
	       put_packet(dp, hd->hd_PacketServerPort, hd->hd_ReplyPort);

	       /* If succeeded ACTION_DIE then leave inner packet loop */
	       if (dp->dp_Res1 == DOSTRUE) {
		  keepon = FALSE;
	       }

	       /* Return packet to sender */
	       return_packet(dp, dp->dp_Res1, dp->dp_Res2);
	    } else {

	       /* Send packet asynchronously to packet server port */
	       PutMsg(hd->hd_PacketServerPort, mn);
	    }
	 }
      }
   } while (quit_flag == FALSE);
}
	/* Create and initialize structure with global data for handler process */

   STATIC struct HandlerData *
create_handler_data(struct Process  *pr, struct DosPacket  *dp)
{
   struct HandlerData  *hd;

   /* Alloc handler data struct and create some ports */
   if (hd = AllocMem(sizeof(struct HandlerData), MEMF_PUBLIC | MEMF_CLEAR)) {
      if (hd->hd_ReplyPort = CreatePort(NULL, 0L)) {
	 if (hd->hd_ZombiePort = CreatePort(NULL, 0L)) {
	    struct DeviceList  *dl;

	    /* Alloc volume node */
	    if (dl = DosAllocMem(sizeof(struct DeviceList))) {
	       struct MsgPort  *pp = &pr->pr_MsgPort;

	       /* Init volume node */
	       dl->dl_Type     = DLT_VOLUME;
	       dl->dl_Task     = pp;
	       dl->dl_Lock     = NULL;
	       dl->dl_LockList = NULL;
	       dl->dl_DiskType = ID_DOS_DISK;
	       dl->dl_Name     = NULL;
	       DateStamp(&dl->dl_VolumeDate);

	       /* Init global handler data */
	       hd->hd_Process    = pr;
	       hd->hd_PacketPort = pp;
	       hd->hd_DeviceNode = (struct DeviceNode *)BTOC(dp->dp_Arg3);
	       hd->hd_VolumeNode = dl;
	       DEBUG2(" create_handler_data: hd=$%lx dl=$%lx", hd, dl);
	       return(hd);
	    }
	    DosFreeMem(dl);
	 }
	 DeletePort(hd->hd_ZombiePort);
      }
      DeletePort(hd->hd_ReplyPort);
   }
   FreeMem(hd, sizeof(struct HandlerData));
   return NULL;
}
	/* Release structure with global data for handler process */

   STATIC VOID
delete_handler_data(struct HandlerData  *hd)
{
   struct DeviceList  *dl = hd->hd_VolumeNode;

   /* Before releasing volume node free volume name */
   DosFreeMem(BTOC(dl->dl_Name));
   DosFreeMem(dl);

   /* Free resources created in handler_process_startup() */
   DeletePort(hd->hd_ZombiePort);
   DeletePort(hd->hd_ReplyPort);

   /* Free resources created in parse_argument_string() */
   DosFreeMem(hd->hd_PackMethod);
   DosFreeMem(hd->hd_Password);
   if (hd->hd_PathLock) {
      UnLock(hd->hd_PathLock);
   }
   FreeMem(hd, sizeof(struct HandlerData));
}
	/* Parse arguments given via startup packet */

   STATIC BOOL
parse_argument_string(struct HandlerData  *hd, BSTR ss)
{
   BYTE   *arg, *buffer;
   USHORT arg_len;
   BOOL   result = FALSE;

   /* Check if user startup string given in dos packet */
   if (!(arg = (BYTE *)BADDR(ss))) {
      arg_len = 0;
   } else {

      /* Remove leading and trailing double qoutes */
      if ((arg_len = (USHORT)*arg++) && *arg == '"') {
	 arg++;
	 if (--arg_len && *(arg + arg_len - 1) == '"') {
	    arg_len--;
	 }
      }
   }

   /* If no startup string given then use default one */
   if (!arg_len) {
      arg     = DEFAULT_ARUMENT_STRING;
      arg_len = strlen(arg);
   }

   /* Alloc buffer for copy of startup string */
   if (buffer = DosAllocMem((LONG)(arg_len + 1))) {
      BYTE *argv[MAX_ARGUMENTS];

      /* Copy argument string by replacing all `_' with ` ' */
      {
	 BYTE   *ptr = arg, *buf = buffer;
	 USHORT len = arg_len;

	 while (len--) {
	    BYTE c;

	    /* Check for '_' */
	    if ((c = *ptr++) == '_') {

	       /* Check for '__' */
	       if (len && *ptr == '_') {
		  ptr++;
		  len--;
		  arg_len--;
		  c = '_';
	       } else {
		  c = ' ';
	       }
	    }
	    *buf++ = c;
	 }
	 *buf = '\0';
      }
      DEBUG2("  parse_argument_string: len=%d arg=`%s'", arg_len, buffer);

      /* Clear argument array */
      {
	 USHORT i;

	 for (i = 0; i < MAX_ARGUMENTS; i++) {
	    argv[i] = NULL;
	 }
      }

      /* Parse copied argument string and check arguments */
      if (GADS(buffer, (LONG)arg_len, NULL, &argv[0], TEMPLATE) >= 0L) {
	 BYTE *ptr;
	 LONG value;

	 /* Get values from argument array */
	 if (ptr = argv[ARGUMENT_ROOT_DIR]) {
	    DEBUG1("  parse_argument_string: root_dir=`%s'", ptr);
	    if (!(hd->hd_PathLock = Lock(ptr, (LONG)SHARED_LOCK))) {
	       error_requester("Can't lock path given in startup string");
	    } else {
	       if (!(ptr = argv[ARGUMENT_VOLUME_NAME])) {
		  ptr = DEFAULT_VOLUME_NAME;
	       }
	       DEBUG1("  parse_argument_string: volume_name=`%s'", ptr);
	       if (hd->hd_VolumeNode->dl_Name = build_bstring(ptr)) {
		  if (ptr = argv[ARGUMENT_AUTO_COMPRESS]) {
		     hd->hd_Flags |= HANDLER_DATA_FLAG_AUTO_COMPRESS;
		  }
		  if (ptr = argv[ARGUMENT_STEP_DOWN]) {
		     hd->hd_Flags |= HANDLER_DATA_FLAG_STEP_DOWN;
		  }
		  if (!(ptr = argv[ARGUMENT_XPK_PRIORITY])) {
		     value = DEFAULT_XPK_PRIORITY;
		  } else {
		     value = Atol(ptr);
		     if (IoErr()) {
			value = DEFAULT_XPK_PRIORITY;
		     }
		  }
		  hd->hd_XpkPriority = value;
		  if (!(ptr = argv[ARGUMENT_PACK_METHOD])) {
		     ptr = DEFAULT_PACK_METHOD;
		  }
		  DEBUG1("  parse_argument_string: pack_mode=`%s'", ptr);
		  if (hd->hd_PackMethod = build_string(ptr)) {
		     if (!(ptr = argv[ARGUMENT_PASSWORD]) ||
				    (hd->hd_Password = build_string(ptr))) {
			DEBUG1("  parse_argument_string: password=`%s'", ptr);
			result = TRUE;
		     }
		  }
	       }
	    }
	 }
      }
   }
   return result;
}
	/* Link volume node into AmigaDOS device list */

   STATIC VOID
link_volume_node(struct DeviceList  *vn)
{
   struct DosInfo  *di;

   /* Link volume node at start of list */
   Forbid();
   di             = (struct DosInfo *)BTOC(((struct RootNode *)
						DOSBase->dl_Root)->rn_Info);
   vn->dl_Next    = di->di_DevInfo;
   di->di_DevInfo = (BPTR)CTOB(vn);
   Permit();
   DEBUG0("  link_volume_node: passed");
}
	/* Unlink volume node from AmigaDOS device list */

   STATIC VOID
unlink_volume_node(struct DeviceList  *vn)
{
   struct DosInfo     *di;
   struct DeviceList  *dl;
   BPTR *dl_ptr;

   /* Search device node in list */
   Forbid();
   di     = (struct DosInfo *)BTOC(((struct RootNode *)
						DOSBase->dl_Root)->rn_Info);
   dl_ptr = &di->di_DevInfo;
   for (dl = (struct DeviceList *)BTOC(*dl_ptr); dl && dl != vn;
			      dl = (struct DeviceList *)BTOC(dl->dl_Next)) {
      dl_ptr = &dl->dl_Next;
   }
   if (dl != vn) {
      error_requester("Can't find volume node");
   } else {

      /* Remove volume node from list */
      *dl_ptr = vn->dl_Next;
   }
   Permit();
   DEBUG0("  unlink_volume_node: passed");
}
