		/*******************************************************
		 *                                                     *
		 *   Sends ACTION_DIE to handler of given device and   *
		 *    removes device node from AmigaDOS device list    *
		 *                                                     *
		 *******************************************************/

	/* Includes */

#include <exec/types.h>
#include <libraries/filehandler.h>
#include <libraries/dosextens.h>
#include <libraries/arpbase.h>
#include <functions.h>
#include <string.h>
#include <fcntl.h>

	/* Defines */

#define MAX_ARGUMENTS		1

#define ARGUMENT_NAME		0

#define MAX_HANDLER_NAME_LEN	32

	/* Globals */

struct ArpBase  *ArpBase;

	/* Externals */

IMPORT struct DosLibrary  *DOSBase;

	/* Statics */

STATIC BYTE *template  = "Device",
	    *xtra_help = "\n<Name> = AmigaDOS name of device "
				    "(with or without colon) to kill\n";
	/* Prototypes */

LONG main(BYTE *arg_line, LONG arg_len);

	/* Pragmas */

#pragma regcall(main(a0,d0))

	/* Main routine - called directly by startup code killcrt0.asm */

   LONG
main(BYTE *arg_line, LONG arg_len)
{
   LONG return_code = RETURN_FAIL;

   /* First open ARP library */
   if (!(ArpBase = OpenLibrary(ArpName, ArpVersion))) {
      Write(Output(), "Can't open arp.library v39+\n", 28L);
   } else {
      BYTE *argv[MAX_ARGUMENTS + 1];

      /* Clear argument string array */
      {
	 USHORT i;

	 for (i = 0; i < MAX_ARGUMENTS; i++) {
	    argv[i] = NULL;
	 }
      }

      /* Parse command line arguments */
      if (GADS(arg_line, arg_len, xtra_help, &argv[0], template) < 0) {
	 Puts((BYTE *)argv[0]);
      } else {
	 BYTE *device_name;

	 /* Check arguments */
	 if (device_name = argv[ARGUMENT_NAME]) {
	    struct DosInfo     *di;
	    struct DeviceNode  *dn;

	    /* Strip colon (if any) from end of device name */
	    {
	       BYTE *ptr;

	       if (ptr = strchr(device_name, ':')) {
		  *ptr = '\0';
	       }
	    }

	    /* Search device node in AmigaDOS device list */
	    Forbid();
	    di = (struct DosInfo *)BADDR(((struct RootNode *)
						DOSBase->dl_Root)->rn_Info);
	    /* Search loop */
	    for (dn = (struct DeviceNode *)BADDR(di->di_DevInfo); dn;
			     dn = (struct DeviceNode *)BADDR(dn->dn_Next)) {
	       if (dn->dn_Type == DLT_DEVICE) {
		  BYTE buffer[MAX_HANDLER_NAME_LEN + 1];

		  /* Compare device names */
		  BtoCStr(buffer, dn->dn_Name, (LONG)MAX_HANDLER_NAME_LEN);
		  if (!Strcmp(buffer, device_name)) {
		     break;
		  }
	       }
	    }
	    Permit();

	    /* Check if appropriate device node found */
	    if (dn && dn->dn_Type != DLT_DIRECTORY) {
	       BOOL remove_flag = TRUE;

	       /* If process exists then terminate handler */
	       if (dn->dn_Task && dn->dn_Handler) {
		  BYTE buffer[MAX_HANDLER_NAME_LEN + 1];

		  /* Send ACTION_DIE to handler process */
		  BtoCStr(buffer, dn->dn_Handler, (LONG)MAX_HANDLER_NAME_LEN);
		  if (dos_packet(dn->dn_Task, (LONG)ACTION_DIE) == DOSFALSE) {
		     Printf("Can't terminate `%s'\n", buffer);
		     remove_flag = FALSE;
		  } else {
		     Printf("`%s' terminated\n", buffer);
		  }
	       }

	       /* Remove device node and release memory */
	       if (remove_flag == TRUE) {
		  struct DeviceNode  *dn_save = dn;
		  BPTR *dn_ptr;

		  /* Search device node in AmigaDOS device list */
		  Forbid();
		  di     = (struct DosInfo *)BADDR(((struct RootNode *)
						DOSBase->dl_Root)->rn_Info);
		  dn_ptr = &di->di_DevInfo;

		  /* Search loop */
		  for (dn = (struct DeviceNode *)BADDR(*dn_ptr); dn;
			     dn = (struct DeviceNode *)BADDR(dn->dn_Next)) {
		     if (dn == dn_save) {
			break;
		     } else {
			dn_ptr = &dn->dn_Next;
		     }
		  }

		  /* Check if appropriate device node found */
		  if (dn) {
		     BYTE *ptr;

		     /* Remove device node from AmigaDOS device list */
		     *dn_ptr = dn->dn_Next;
		     Printf("Device node for `%s' removed\n", device_name);

		     /* Release BSTR with handler name */
		     if (ptr = (BYTE *)BADDR(dn->dn_Handler)) {
			DosFreeMem(ptr);
		     }

		     /* Release BSTR with device name */
		     if (ptr = (BYTE *)BADDR(dn->dn_Name)) {
			DosFreeMem(ptr);
		     }

		     /* Release device node */
		     DosFreeMem(dn);
		  }
		  Permit();
		  return_code = RETURN_OK;
	       }
	    }
	 }
      }
      CloseLibrary(ArpBase);
   }
   return return_code;
}
