		/**************************************
		 *                                    *
		 *   AmigaDOS Handler for XPK files   *
		 *                                    *
		 *       Packet server routines       *
		 *                                    *
		 **************************************/

	/* Includes */

#include "includes.h"
#include "defines.h"
#include "protos.h"

	/* Startup packet server process */

   VOID
packet_server_process_startup(VOID)
{
   struct Process       *pr;
   struct MsgPort       *pp;
   struct HandshakeMsg  *hm;
   struct PacketData    *pd;	/* global data for packet server process */
   BOOL result;

   /* Initialize data base register for small code/data memory model */
   geta4();
   result = FALSE;

   /* Wait for handshake msg arriving at our process message port */
   pr = (struct Process *)FindTask(NULL);
   pp = &pr->pr_MsgPort;
   DEBUG2("packet_server_process_startup: pr=$%lx   pp=$%lx", pr, pp);
   WaitPort(pp);
   hm = (struct HandshakeMsg *)GetMsg(pp);
   DEBUG2("packet_server_process_startup: hm=$%lx   hd=$%lx", hm,
							       hm->hm_Data);
   /* Allocate resources needed for packet server process */
   if (pd = AllocMem(sizeof(struct PacketData), MEMF_PUBLIC | MEMF_CLEAR)) {

      /* We have to open ARP library here again to start ARP resource
       * tracking for server process too
       */
      if (pd->pd_ArpBase = OpenLibrary(ArpName, ArpVersion)) {
	 if (pp = CreatePort(NULL, 0L)) {

	    /* Init global data struct */
	    pd->pd_PacketPort     = pp;
	    pd->pd_HandlerData    = (struct HandlerData *)hm->hm_Data;
	    pd->pd_HandleList     = NULL;
	    pd->pd_LockList       = NULL;
	    pd->pd_Flags          = 0L;

	    /* Reply startup message with our msg port */
	    hm->hm_Data = (LONG)pp;
	    ReplyMsg((struct Message *)hm);

	    /* Call packet server process main routine */
	    packet_server_process_main(pd);
	    result = TRUE;

	    /* Free resources and fall through -> ARP send zombie message
	     * to handler process
	     */
	    DeletePort(pp);
	 }
	 CloseLibrary(pd->pd_ArpBase);
      }
      FreeMem(pd, sizeof(struct PacketData));
   }

   /* If startup failed then reply startup message with result */
   if (result == FALSE) {
      hm->hm_Data = NULL;
      ReplyMsg((struct Message *)hm);
   }
   DEBUG1("packet_server_process_startup: result=%d", result);
}
	/* Packet server process main routine */

   STATIC VOID
packet_server_process_main(struct PacketData  *pd)
{
   struct MsgPort  *pp = pd->pd_PacketPort;
   BOOL quit_flag = FALSE;

   /* Server packet loop */
   do {
      struct Message  *mn;

      /* Wait for AmigaDOS packet */
      WaitPort(pp);

      /* Perform all packets */
      while (mn = GetMsg(pp)) {
	 struct DosPacket  *dp = (struct DosPacket *)mn->mn_Node.ln_Name;
	 LONG res1, error;

	 /* Set default results for success */
	 res1  = DOSTRUE;
	 error = 0L;

	 /* Check packet action type */
	 switch (dp->dp_Type) {
	    case ACTION_FINDUPDATE :   /* FileHandle, Lock, Name -> BOOL */
	       
	       /* Packet: open stream for input and output (not supported) */
	       DEBUG2("  ACTION_FINDSOMETHING (%ld): Open(%s)",
			      dp->dp_Type, ((BYTE *)BTOC(dp->dp_Arg3)) + 1);
	       error = ERROR_ACTION_NOT_KNOWN;
	       break;

	    case ACTION_END :   /* fh_Arg1 -> BOOL */

	       /* Packet: close current stream */
	       DEBUG1("  ACTION_END: fh_Arg1=$%lx", dp->dp_Arg1);
	       error = close_file(pd, dp);
	       break;

	    case ACTION_FINDINPUT :   /* FileHandle, Lock, Name -> BOOL */
	    case ACTION_FINDOUTPUT :   /* FileHandle, Lock, Name -> BOOL */

	       /* Packet: open stream for input and/or output */
	       DEBUG2("  ACTION_FINDSOMETHING (%ld): Open(%s)",
			      dp->dp_Type, ((BYTE *)BTOC(dp->dp_Arg3)) + 1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg2))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = open_file(pd, dp);
		  }
	       }
	       break;

	    case ACTION_READ :   /* fh_Arg1, Buffer, Length -> ActualLength */

	       /* Packet: read data from stream */
	       DEBUG3("  ACTION_READ: fh_Arg1=$%lx buffer=$%lx len=%ld",
				     dp->dp_Arg1, dp->dp_Arg2, dp->dp_Arg3);
	       error = read_file(pd, dp, &res1);
	       break;

	    case ACTION_WRITE :   /* fh_Arg1, Buffer, Length -> ActualLength */

	       /* Packet: write data to stream */
	       DEBUG3("  ACTION_WRITE: fh_Arg1=$%lx buffer=$%lx len=%ld",
				     dp->dp_Arg1, dp->dp_Arg2, dp->dp_Arg3);
	       error = write_file(pd, dp, &res1);
	       break;

	    case ACTION_SEEK :   /* fh_Arg1, Position, Mode -> OldPosition */

	       /* Packet: seek to new position in file */
	       DEBUG3("  ACTION_SEEK: fh_Arg1=$%lx offset=%ld mode=%ld",
				     dp->dp_Arg1, dp->dp_Arg2, dp->dp_Arg3);
	       error = seek_file(pd, dp, &res1);
	       break;

	    case ACTION_CURRENT_VOLUME :   /* fh_Arg1 -> VolumeNode, UnitNumber */

	       /* Packet: get volume node of stream */
	       DEBUG1("  ACTION_CURRENT_VOLUME: fh_Arg1=$%lx", dp->dp_Arg1);
	       if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		  error = ERROR_NOT_A_DOS_DISK;
	       } else {
		  error = get_current_volume(pd, dp, &res1);
	       }
	       break;

	    case ACTION_LOCATE_OBJECT :   /* Lock, Name, Mode -> Lock */

	       /* Packet: lock object in given mode */
	       DEBUG2("  ACTION_LOCATE_OBJECT: Lock(%s, %ld)",
			      ((BYTE *)BTOC(dp->dp_Arg2)) + 1, dp->dp_Arg3);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = create_internal_lock(pd, dp, &res1);
		  }
	       }
	       break;

	    case ACTION_FREE_LOCK :   /* Lock -> BOOL */

	       /* Packet: unlock given object */
	       DEBUG1("  ACTION_FREE_LOCK: UnLock($%lx)", dp->dp_Arg1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  error = delete_internal_lock(pd, dp);
	       }
	       break;

	    case ACTION_SAME_LOCK :   /* Lock1, Lock2 -> BOOL */

	       /* Packet: unlock given object */
	       DEBUG2("  ACTION_SAME_LOCK: SameLock($%lx, $%lx)",
						  dp->dp_Arg1, dp->dp_Arg2);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1)) &&
			  !(error = check_internal_lock(pd, dp->dp_Arg2))) {
		  error = same_lock(pd, dp, &res1);
	       }
	       break;
		
	    case ACTION_COPY_DIR :   /* Lock -> Lock */

	       /* Packet: duplicate given lock */
	       DEBUG1("  ACTION_COPY_DIR: duplicate lock $%lx", dp->dp_Arg1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  error = duplicate_internal_lock(pd, dp, &res1);
	       }
	       break;

	    case ACTION_EXAMINE_OBJECT :   /* Lock, FileInfoBlock -> BOOL */

	       /* Packet: get infos about locked object */
	       DEBUG2("  ACTION_EXAMINE_OBJECT: Examine($%lx,$%lx)",
						  dp->dp_Arg1, dp->dp_Arg2);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = get_file_info_block(pd, dp);
		  }
	       }
	       break;

	    case ACTION_EXAMINE_NEXT :   /* Lock, FileInfoBlock -> BOOL */

	       /* Packet: get infos about next entry in directory */
	       DEBUG2("  ACTION_EXAMINE_NEXT: ExNext($%lx,$%lx)",
						  dp->dp_Arg1, dp->dp_Arg2);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = get_file_info_block(pd, dp);
		  }
	       }
	       break;

	    case ACTION_PARENT :   /* Lock -> Lock */

	       /* Packet: get lock to parent directory */
	       DEBUG1("  ACTION_PARENT: ParentDir($%lx)", dp->dp_Arg1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = parent_internal_lock(pd, dp, &res1);
		  }
	       }
	       break;

	    case ACTION_DISK_INFO :   /* InfoData -> BOOL:TRUE */

	       /* Packet: get lock to parent directory */
	       DEBUG1("  ACTION_DISK_INFO: InfoData=$%lx", dp->dp_Arg1);
	       error = get_info_data(pd, dp);
	       break;

	    case ACTION_INFO :   /* Lock, InfoData -> BOOL */

	       /* Packet: get lock to parent directory */
	       DEBUG2("  ACTION_INFO: Info($%lx,$%lx)", dp->dp_Arg1,
							       dp->dp_Arg2);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = get_info_data(pd, dp);
		  }
	       }
	       break;

	    case ACTION_CREATE_DIR :   /* Lock, Name -> Lock */

	       /* Packet: create new directory */
	       DEBUG1("  ACTION_CREATE_DIR: MakeDir(%s)",
					   ((BYTE *)BTOC(dp->dp_Arg2)) + 1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = create_dir(pd, dp, &res1);
		  }
	       }
	       break;

	    case ACTION_DELETE_OBJECT :   /* Lock, Name -> BOOL */

	       /* Packet: delete given object */
	       DEBUG1("  ACTION_DELETE_OBJECT: Delete(%s)",
					   ((BYTE *)BTOC(dp->dp_Arg2)) + 1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = delete_object(pd, dp);
		  }
	       }
	       break;

	    case ACTION_RENAME_OBJECT :   /* SLock, SName, DLock, DName -> BOOL */

	       /* Packet: rename given object */
	       DEBUG2("  ACTION_RENAME_OBJECT: Rename(%s,%s)",
					    ((BYTE *)BTOC(dp->dp_Arg2)) + 1,
					   ((BYTE *)BTOC(dp->dp_Arg4)) + 1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg1)) &&
			  !(error = check_internal_lock(pd, dp->dp_Arg3))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = rename_object(pd, dp);
		  }
	       }
	       break;

	    case ACTION_SET_PROTECT :   /* -, Lock, Name, Mask -> BOOL */

	       /* Packet: set protection on given object */
	       DEBUG2("  ACTION_SET_PROTECT: SetProtect(%s,$%lx)",
			      ((BYTE *)BTOC(dp->dp_Arg3)) + 1, dp->dp_Arg4);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg2))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = set_something(pd, dp);
		  }
	       }
	       break;

	    case ACTION_SET_COMMENT :   /* -, Lock, Name, Mask -> BOOL */

	       /* Packet: set comment on given object */
	       DEBUG2("  ACTION_SET_COMMENT: SetComment(%s,%s)",
					    ((BYTE *)BTOC(dp->dp_Arg3)) + 1,
					   ((BYTE *)BTOC(dp->dp_Arg4)) + 1);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg2))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = set_something(pd, dp);
		  }
	       }
	       break;

	    case ACTION_SET_DATE :   /* -, Lock, Name, DateStamp -> BOOL */

	       /* Packet: set date on given object */
	       DEBUG2("  ACTION_SET_DATE: SetDate(%s,$%lx)",
			      ((BYTE *)BTOC(dp->dp_Arg3)) + 1, dp->dp_Arg4);
	       if (!(error = check_internal_lock(pd, dp->dp_Arg2))) {
		  if (pd->pd_Flags & PACKET_DATA_FLAG_INHIBIT) {
		     error = ERROR_NOT_A_DOS_DISK;
		  } else {
		     error = set_something(pd, dp);
		  }
	       }
	       break;

	    case ACTION_INHIBIT :   /* -> BOOL */

	       /* Packet: lock/unlock handler */
	       DEBUG1("  ACTION_INHIBIT: mode=%ld", dp->dp_Arg1);

	       /* Change inhibit flag according to given BOOL */
	       if (dp->dp_Arg1 == DOSTRUE) {
		  pd->pd_Flags |= PACKET_DATA_FLAG_INHIBIT;
	       } else {
		  pd->pd_Flags &= ~PACKET_DATA_FLAG_INHIBIT;
	       }
	       break;

	    case ACTION_IS_FILESYSTEM :   /* -> BOOL */

	       /* Packet: check if handler is a filesystem */
	       DEBUG0("  ACTION_IS_FILESYSTEM");

	       /* Yes we are a filesystem, so return default results */
	       break;

	    case ACTION_DIE :   /* -> BOOL */

	       /* Packet: terminate handler */
	       DEBUG0("  ACTION_DIE");

	       /* Check for open internal handles, locks and IFF servers */
	       if (pd->pd_HandleList || pd->pd_LockList) {
		  res1  = DOSFALSE;
		  error = ERROR_OBJECT_IN_USE;
	       } else {

		  /* Before quitting server process clear packet port */
		  pd->pd_HandlerData->hd_PacketServerPort = NULL;
		  quit_flag = TRUE;
	       }

	       /* Save results in dos packet and return it syncronously */
	       dp->dp_Res1 = res1;
	       dp->dp_Res2 = error;
	       ReplyMsg(mn);

	       /* Don't return dos packet */
	       dp = NULL;
	       break;

	    default :

	       /* Packet: unknown packet - reject it */
	       DEBUG1("  unknown action=%ld", dp->dp_Type);
	       error = ERROR_ACTION_NOT_KNOWN;
	       break;
	 }

	 /* If packet not replied then return it now */
	 if (dp) {

	    /* Get results - for ACTION_CURRENT_VOLUME results used otherwise */
	    if (dp->dp_Type != ACTION_CURRENT_VOLUME) {

	       /* Check for error and default positive primary result */
	       if (error && res1 == DOSTRUE) {

		  /* Set default negative primary result */
		  switch (dp->dp_Type) {
		     case ACTION_READ :
		     case ACTION_WRITE :
			res1 = 0L;
			break;

		     case ACTION_SEEK :
			res1 = -1L;
			break;

		     case ACTION_LOCATE_OBJECT :
		     case ACTION_COPY_DIR :
		     case ACTION_PARENT :
			res1 = NULL;
			break;

		     default :
			res1 = DOSFALSE;
			break;
		  }
	       }
	    }

	    /* Return dos packet with results asyncronously */
	    DEBUG3("  packet_server_process_main: dp=$%lx res1=%ld "
					      "error=%ld", dp, res1, error);
	    return_packet(dp, res1, error);
	 }
      }
   } while (quit_flag == FALSE);
}
